/**
  PasswordMaker - Creates and manages passwords
  Copyright (C) 2005 Eric H. Jung and LeahScape, Inc.
  http://passwordmaker.org/
  grimholtz@yahoo.com

  This library is free software; you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by
  the Free Software Foundation; either version 2.1 of the License, or (at
  your option) any later version.

  This library is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESSFOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
  for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this library; if not, write to the Free Software Foundation,
  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
**/

var accountsTree, accountsDeck;

function initAccountsPanel() {

  accountsTree = document.getElementById("accountsTree");
  accountsDeck = document.getElementById("accountsDeck");

  accountsTree.database.AddDataSource(passwordMakerHelper.datasource);
  accountsTree.builder.rebuild();
  openFolders(accountsTree);
  accountsTree.view.selection.select(0);

  //accountsTree.oncommand=cancelEvent;
  //accountsTree.oninput=cancelEvent;
  //accountsTree.onkeypress=cancelEvent;
  //accountsTree.onclick=cancelEvent;
  //var wnd = document.getElementById("passwordmakerdialog");
  //wnd.oncommand = onAdvancedOptionsChange;
  //wnd.oninput = onAdvancedOptionsChange;
  //wnd.onkeypress = onAdvancedOptionsChange;
//  wnd.onclick = cancelEvent;
}

function cancelEvent(evt) {
  dump(evt.type + "\n");
  evt.preventDefault();
  evt.stopPropagation();
  return false;
}

function openFolders(tree) {
  var j = tree.view.rowCount;
  var openMe = new Array();
  for (i=0; i<j; i++) {
    if (tree.view.isContainer(i) && !tree.view.isContainerOpen(i))
      openMe.push(i);
  }
  // Must open in reverse otherwise the index is screwed up
  for (var k=openMe.length; k>0; k--) {
    try {
      tree.view.toggleOpenState(k);
    }
    catch (e) {
      // Calling toggleOpenState() on folders with no children
      // causes an exception. Just catch it and move on.
    }
  }
}

function onAccountsTreeSelection() {
  // Disable delete button if there's no selection or the "defaults" row is selected.
  if (accountsTree.currentIndex == -1) {
    // Nothing selected. select the first item
    accountsTree.view.selection.select(0);
    return;
  }
  defaultAccountOptionsSelected = accountsTree.currentIndex == 0;
  deleteTreeFolderOrItemBtn.disabled = defaultAccountOptionsSelected;

  // If not a folder, generate password
  if (accountsTree.view.getItemAtIndex(accountsTree.currentIndex).getAttribute("container") == "true") {
    newAccountBtn.disabled = false; // Allow creation of new accounts
    usingURLLabel.value = usingURLPrefixLabel.value = passwordGeneratedLabel.value = passwordGenerated.value = "";
  }
  else {
    newAccountBtn.disabled = true; // Disallow creation of new accounts
    updateURLLabel();
    passwordGeneratedLabel.value = "Generated Password";
    generatepassword();
  }
}

function promptForTwoItems(nameLabel, descLabel) {
   var params = {inn: {name:nameLabel, description:descLabel}, out:null};

   window.openDialog("chrome://passwdmaker/chrome/treeprompt.xul",
      "", "chrome, dialog, modal, resizable=yes", params).focus();

  if (params.out) {
    // User clicked OK. 
    while (params.out != null && params.out.folderName == "") {
      // MUST enter a folder name
      passwordMakerHelper.prompts.alert(this, "PasswordMaker", "A name is required.");
      params.out = null;
      window.openDialog("chrome://passwdmaker/chrome/treeprompt.xul", "", "chrome, dialog, modal, resizable=yes",
        params).focus();
    }
    return params.out;
  }
}

function createNewFolder(folderNameURI, folderDescURI, containerSeq) {
  var folderInfo = promptForTwoItems("Folder Name (required)", "Folder Description (optional)");
  if (!folderInfo)
    return; // user cancelled

  // Build the folder resource...
  // Folder Name
  var folderRes = passwordMakerHelper.rdfService.GetAnonymousResource();
  _updateDatasourceTarget(folderRes, folderNameURI, folderInfo.folderName);

  // Folder Description
  _updateDatasourceTarget(folderRes, folderDescURI, folderInfo.folderDescription);

  // Make the (empty) sequence, and append it to the container (e.g., the accounts sequence)
  var newContainer = passwordMakerHelper.rdfContainerUtils.MakeSeq(passwordMakerHelper.datasource, folderRes);
  containerSeq.AppendElement(folderRes);
  
  passwordMakerHelper.datasourceFlusher.Flush();
}

function onCreateNewFolder() {
  createNewFolder("name", "description", passwordMakerHelper.accountsContainer);
}

/**
 * Called to create a new account in the currently
 * selected folder.
 */
function onCreateNewAccount() {
  var folderRes = getSelectedItemAsResource(accountsTree);
  if (!folderRes)
    return;
  var containerRes = passwordMakerHelper.rdfContainerUtils.MakeSeq(passwordMakerHelper.datasource, folderRes);

  var accountRes = passwordMakerHelper.rdfService.GetAnonymousResource();
  // Define the resources
  _updateDatasourceTarget(accountRes, "name", "New Account");
  _updateDatasourceTarget(accountRes, "description", "None");
  _updateDatasourceTarget(accountRes, "whereLeetLB", "off");
  _updateDatasourceTarget(accountRes, "leetLevelLB", "9");
  _updateDatasourceTarget(accountRes, "hashAlgorithmLB", "sha256");
  _updateDatasourceTarget(accountRes, "passwordLength", "8");
  _updateDatasourceTarget(accountRes, "urlToUse", "");
  _updateDatasourceTarget(accountRes, "urlToMatch", "");
  _updateDatasourceTarget(accountRes, "usernameTB", "");
  _updateDatasourceTarget(accountRes, "counter", "");
  _updateDatasourceTarget(accountRes, "charset", "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789`~!@#$%^&*()_-+={}|[]\\:\";\'<>?,./");
  _updateDatasourceTarget(accountRes, "prefix", "");
  _updateDatasourceTarget(accountRes, "suffix", "");

  // Add the resource to the container (folder)
  containerRes.AppendElement(accountRes);
  passwordMakerHelper.datasourceFlusher.Flush();

  // Open the selected item (the parent folder of the new accounts we just created)
  // if it's not already open.
  if (!accountsTree.view.isContainerOpen(accountsTree.currentIndex))
    accountsTree.view.toggleOpenState(accountsTree.currentIndex);

  // @TODO: Select the last item in the now-open folder (the newly-created account)
}

/**
 * Called to update a single target.
 */
function _updateDatasourceTarget(subjectRes, predicateStr, literalStr) {
  var newTargetRes = passwordMakerHelper.rdfService.GetLiteral(literalStr);
  var predicateRes = passwordMakerHelper.rdfService.GetResource("http://passwordmaker.mozdev.org/rdf#" + predicateStr);
  var oldTargetRes = passwordMakerHelper.datasource.GetTarget(subjectRes, predicateRes, true);
  try {
    if (oldTargetRes == null) {
      passwordMakerHelper.datasource.Assert(subjectRes, predicateRes, newTargetRes, true);
    }
    else {
      passwordMakerHelper.datasource.Change(subjectRes, predicateRes, oldTargetRes, newTargetRes);
    }
  }
  catch (e) {
    dump(predicateStr + "\n");
    dump(e + "\n");
  }
}

/**
 * Return the RDF resource associated with the
 * selected tree item.
 */
function getSelectedItemAsResource(tree) {
  // Get id of selected item in tree
  if (tree.currentIndex > -1) {
    var subjectId = tree.builderView.getItemAtIndex(tree.currentIndex).getAttribute("id"); // e.g., rdf:#$JZPes
    return passwordMakerHelper.rdfService.GetResource(subjectId);
  }
  return null;
}

function deleteTreeItem(tree, container) {
  var copy = container;

  var subject = getSelectedItemAsResource(tree);
  if (!subject)
    return;

  var isContainer = passwordMakerHelper.rdfContainerUtils.IsContainer(passwordMakerHelper.datasource, subject);
  if (!passwordMakerHelper.ask(this, isContainer?"Are you sure you want to delete this item and all items below it?":"Are you sure you want to delete this item?"))
    return;

  if (isContainer) {
    // Subject is a container (folder)
    var container = passwordMakerHelper.rdfContainerUtils.MakeSeq(passwordMakerHelper.datasource, subject);
    var children = container.GetElements();
    while (children.hasMoreElements()){
      var child = children.getNext();
      if (child instanceof Components.interfaces.nsIRDFResource){
        container.RemoveElement(child, true);
        children = container.GetElements();
        // Remove all tuples
        var targets = passwordMakerHelper.datasource.ArcLabelsOut(child);
        while (targets.hasMoreElements()){
          var predicate = targets.getNext();
          if (predicate instanceof Components.interfaces.nsIRDFResource){
            var target = passwordMakerHelper.datasource.GetTarget(child, predicate, true);
            passwordMakerHelper.datasource.Unassert(child, predicate, target);
          }
        }
      }
    }
  }
  else {
    // Not a container (folder). Remove reference
    // to this subject in whichever container contains it.
    var arcsIn = passwordMakerHelper.datasource.ArcLabelsIn(subject);
    while (arcsIn.hasMoreElements()) {
      var arc = arcsIn.getNext();
      if (arc instanceof Components.interfaces.nsIRDFResource) {
        if (passwordMakerHelper.rdfContainerUtils.IsOrdinalProperty(arc)) {
          var parentRes = passwordMakerHelper.datasource.GetSource(arc, subject, true);
          var parentResContainer = passwordMakerHelper.rdfContainerUtils.MakeSeq(passwordMakerHelper.datasource, parentRes);
          parentResContainer.RemoveElement(subject, true);
        }
      }
    }
  }

  // Remove all remaining tuples
  var targets = passwordMakerHelper.datasource.ArcLabelsOut(subject);
  while (targets.hasMoreElements()){
    var predicate = targets.getNext();
    if (predicate instanceof Components.interfaces.nsIRDFResource){
      var target = passwordMakerHelper.datasource.GetTarget(subject, predicate, true);
      passwordMakerHelper.datasource.Unassert(subject, predicate, target);
    }
  }
  copy.RemoveElement(subject, true); // remove from the container
  passwordMakerHelper.datasourceFlusher.Flush();
}

function onDeleteTreeItem() {
  deleteTreeItem(accountsTree, passwordMakerHelper.accountsContainer);
}

function getUrlToUse() {
    var cells = getSelectedTreeCells(accountsTree);
    return accountsTree.currentIndex == 0 ?
      
      // Default Options
      passwordMaker.calculateURI(cells[6].getAttribute("label") == "true", // protocolCB
        cells[7].getAttribute("label") == "true", // subdomainCB
        cells[8].getAttribute("label") == "true", // domainCB
        cells[9].getAttribute("label") == "true", // pathCB
        cells[9].getAttribute("label") == "true") : // pathCB
      
      // A specific account
      cells[11].getAttribute("label");
}

/*
  Returns the following array...

  0 : <treecell field="accountName" label="rdf:http://passwordmaker.mozdev.org/rdf#name"/>
  1 : <treecell field="accountDescription" label="rdf:http://passwordmaker.mozdev.org/rdf#description"/>
  2 : <treecell field="whereLeetLB" label="rdf:http://passwordmaker.mozdev.org/rdf#whereLeetLB"/>
  3 : <treecell field="leetLevelLB" label="rdf:http://passwordmaker.mozdev.org/rdf#leetLevelLB"/>
  4 : <treecell field="hashAlgorithmLB" label="rdf:http://passwordmaker.mozdev.org/rdf#hashAlgorithmLB"/>
  5 : <treecell field="passwordLength" label="rdf:http://passwordmaker.mozdev.org/rdf#passwordLength"/>
  6 : <treecell field="protocolCB" label="rdf:http://passwordmaker.mozdev.org/rdf#protocolCB"/>
  7 : <treecell field="subdomainCB" label="rdf:http://passwordmaker.mozdev.org/rdf#subdomainCB"/>
  8 : <treecell field="domainCB" label="rdf:http://passwordmaker.mozdev.org/rdf#domainCB"/>
  9 : <treecell field="pathCB" label="rdf:http://passwordmaker.mozdev.org/rdf#pathCB"/>
  10 : <treecell field="urlToMatch" label="rdf:http://passwordmaker.mozdev.org/rdf#urlToMatch"/>
  11 : <treecell field="urlToUse" label="rdf:http://passwordmaker.mozdev.org/rdf#urlToUse"/>
  12 : <treecell field="usernameTB" label="rdf:http://passwordmaker.mozdev.org/rdf#usernameTB"/>
  13 : <treecell field="counter" label="rdf:http://passwordmaker.mozdev.org/rdf#counter"/>
  14 : <treecell field="charset" label="rdf:http://passwordmaker.mozdev.org/rdf#charset"/>
*/
function getSelectedTreeCells(tree) {
  var selectedTreeItem = tree.view.getItemAtIndex(tree.currentIndex);   // <treeitem/>
  var selectedTreeRow = selectedTreeItem.firstChild;   // <treerow/>
  return selectedTreeRow.childNodes;  // <treecell/> NodeList
}

function onAccountSettings() {
  /*if (PasswordMakerHelper._masterPassword) {
    var mpw = passwordMakerHelper.getDecryptedMasterPassword();
    if (!mpw || mpw != passwordMaster.value) {
      return;
    }
  }*/
  var subjectRes = getSelectedItemAsResource(accountsTree);
  if (!subjectRes)
    return;

  var isDefaultsAccount = accountsTree.currentIndex == 0;
  var isFolder = accountsTree.view.getItemAtIndex(accountsTree.currentIndex).getAttribute("container") == "true";

  // Get current settings from the UI (we could get them from the RDF instead)
  var cells = getSelectedTreeCells(accountsTree);
  var accountSettings = {isFolder : isFolder, isDefaultsAccount : isDefaultsAccount,
    accountName : cells[0].getAttribute("label"), 
    accountDescription : cells[1].getAttribute("label"), whereLeetLB : cells[2].getAttribute("label"),
    leetLevelLB : cells[3].getAttribute("label"), hashAlgorithmLB : cells[4].getAttribute("label"),
    passwordLength : cells[5].getAttribute("label"), protocolCB : cells[6].getAttribute("label"),
    subdomainCB : cells[7].getAttribute("label"), domainCB : cells[8].getAttribute("label"),
    pathCB : cells[9].getAttribute("label"), urlToMatch : (isDefaultsAccount ? getUrlToUse() : cells[10].getAttribute("label")),
    urlToUse : (isDefaultsAccount ? getUrlToUse() : cells[11].getAttribute("label")), usernameTB : cells[12].getAttribute("label"),
    counter : cells[13].getAttribute("label"), charset : cells[14].getAttribute("label"), prefix : cells[15].getAttribute("label"),
    suffix : cells[16].getAttribute("label")};

  var params = {inn:{accountSettings : accountSettings, passwordMakerHelper : passwordMakerHelper,
    passwordMaster : passwordMaster.value, passwordMaker : passwordMaker}, out : null};

  window.openDialog("chrome://passwdmaker/chrome/accountsettings.xul", "", "chrome, dialog, modal, resizable=yes", params).focus();

  if (params.out) {
    // User clicked OK - update datasource
    accountSettings = params.out.accountSettings;
    _updateDatasourceTarget(subjectRes, "name", accountSettings.accountName);
    _updateDatasourceTarget(subjectRes, "description", accountSettings.accountDescription);
    if (!isFolder) {
      // This isn't a folder -- continue saving the other attributes
      _updateDatasourceTarget(subjectRes, "whereLeetLB", accountSettings.whereLeetLB);
      _updateDatasourceTarget(subjectRes, "leetLevelLB", accountSettings.leetLevelLB);
      _updateDatasourceTarget(subjectRes, "hashAlgorithmLB", accountSettings.hashAlgorithmLB);
      _updateDatasourceTarget(subjectRes, "passwordLength", accountSettings.passwordLength);
      if (!isDefaultsAccount) {
        // Especially don't write urlToMatch, otherwise the Default will match weird stuff!
        _updateDatasourceTarget(subjectRes, "urlToMatch", accountSettings.urlToMatch); 
        _updateDatasourceTarget(subjectRes, "urlToUse", accountSettings.urlToUse);
      }
      _updateDatasourceTarget(subjectRes, "usernameTB", accountSettings.usernameTB);
      _updateDatasourceTarget(subjectRes, "counter", accountSettings.counter);
      _updateDatasourceTarget(subjectRes, "charset", accountSettings.charset);
      _updateDatasourceTarget(subjectRes, "prefix", accountSettings.prefix);
      _updateDatasourceTarget(subjectRes, "suffix", accountSettings.suffix);
      if (isDefaultsAccount) {
        // The defaults node is selected
        _updateDatasourceTarget(subjectRes, "protocolCB", accountSettings.protocolCB ? "true" : "false");
        _updateDatasourceTarget(subjectRes, "subdomainCB", accountSettings.subdomainCB ? "true" : "false");
        _updateDatasourceTarget(subjectRes, "domainCB", accountSettings.domainCB ? "true" : "false");
        _updateDatasourceTarget(subjectRes, "pathCB", accountSettings.pathCB ? "true" : "false");
      }
    }
    params = null; // proactive clearing of sensitive data
    passwordMakerHelper.datasourceFlusher.Flush(); // gen password AFTER flushing so tree is up-to-date for generation
    if (!isFolder) {
      updateURLLabel();
      generatepassword();
    }
  }
}
