/**
  PasswordMaker - Creates and manages passwords
  Copyright (C) 2005 Eric H. Jung and LeahScape, Inc.
  http://passwordmaker.org/
  grimholtz@yahoo.com

  This library is free software; you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by
  the Free Software Foundation; either version 2.1 of the License, or (at
  your option) any later version.

  This library is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESSFOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
  for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this library; if not, write to the Free Software Foundation,
  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
**/

// References to the UI elements
var clearClipboardCB, clearClipboard, autoPopulateCB, maskGeneratedPasswordCB, pwShortcutLB, maskMPWFieldCB, mpwDeck;

function initGlobalSettingsPanel() {

  // Get references to the UI elements
  clearClipboardCB = document.getElementById("clearClipboardCB");
  clearClipboard = document.getElementById("clearClipboard");
  autoPopulateCB = document.getElementById("autoPopulateCB");
  maskGeneratedPasswordCB = document.getElementById("maskGeneratedPasswordCB");
  pwShortcutLB = document.getElementById("pwShortcutLB");
  maskMPWFieldCB = document.getElementById("maskMPWFieldCB");
  mpwDeck = document.getElementById("mpwDeck");

  // Masking
  maskGeneratedPasswordCB.checked = passwordMaker._getLiteral(passwordMaker.globalSettingsSubjectRes, passwordMaker.maskMasterPasswordPredicateRes) == "true";
  
  // Auto-clear clipboard
  clearClipboardCB.checked = passwordMaker._getLiteral(passwordMaker.globalSettingsSubjectRes, passwordMaker.autoClearClipboardPredicateRes) == "true";

  // Auto-clear clipboard secs
  clearClipboard.value = passwordMaker._getLiteral(passwordMaker.globalSettingsSubjectRes, passwordMaker.autoClearClipboardSecondsPredicateRes);

  // Auto-populate pw fields
  autoPopulateCB.checked = passwordMaker._getLiteral(passwordMaker.globalSettingsSubjectRes, passwordMaker.autoPopulatePredicateRes) == "true";

  // Password Shortcut (ALT-`)
  pwShortcutLB.value = passwordMaker._getLiteral(passwordMaker.globalSettingsSubjectRes, passwordMaker.passwordShortcutPredicateRes);
  maskMPWFieldCB.checked = passwordMaker._getLiteral(passwordMaker.globalSettingsSubjectRes, passwordMaker.maskMPWFieldPredicateRes) == "true";
  if (passwordGenerated.value == "") {
    // Can't ever be checked if the mpw is empty
    maskMPWFieldCB.checked = false;
    passwordMakerHelper.saveGlobalSetting("maskMasterPassword", "false");
    passwordMakerHelper.datasourceFlusher.Flush();
  }
  if (!maskMPWFieldCB.checked)
    mpwDeck.selectedIndex = 1;

  onClearClipboardCB();
}

function onMaskMPWField(fromBtn) {
  if (fromBtn) {
    // User wants to turn off masking
    showMPWField();
    return;
  }
  if (maskMPWFieldCB.checked) {
    if (passwordMaster.value == "") {
      passwordMakerHelper.prompts.alert(this, "PasswordMaker", "You must first enter a master password");
      maskMPWFieldCB.checked = false;
    }
    else
      mpwDeck.selectedIndex = 0;
  }
  else
    showMPWField();
  passwordMakerHelper.saveGlobalSetting("maskMasterPassword", maskMPWFieldCB.checked ? "true":"false");
  passwordMakerHelper.datasourceFlusher.Flush();
}

function showMPWField() {
  // User wants to turn off masking
  // Firefox unchecked the checbkox, even though the user
  // hasn't entered the master pw yet. Check it here so
  // the UI matches the current state of affairs.
  maskMPWFieldCB.checked = true;
  if (passwordMakerHelper.prompwMPW(passwordMaster.value)) {
    maskMPWFieldCB.checked = false;
    mpwDeck.selectedIndex = 1;
  }
}

/**
 * Method which fires when the Clear Clipboard checkbox
 * has been clicked.
 */
function onClearClipboardCB() {
  clearClipboard.disabled = !clearClipboardCB.checked;
  if (clearClipboardCB.checked && clearClipboard.value == "")
    clearClipboard.value = "10";
  passwordMakerHelper.saveGlobalSetting("autoClearClipboard", clearClipboardCB.checked ? "true":"false");
  passwordMakerHelper.datasourceFlusher.Flush();
}

function onClearClipboard() {
  if (/\D/.test(clearClipboard.value))
    clearClipboard.value="10";
  passwordMakerHelper.saveGlobalSetting("autoClearClipboardSeconds", clearClipboard.value);
  passwordMakerHelper.datasourceFlusher.Flush();
}

function onAutoPopulateCB() {
  passwordMakerHelper.saveGlobalSetting("autoPopulate", autoPopulateCB.checked ? "true":"false");
  passwordMakerHelper.datasourceFlusher.Flush();
  // Now start looking for password input boxes if autoPopulate is true. If false, stop looking for them.
  // But first ensure the user hasn't closed the opening browser since this window is non-modal.
  if (window.opener && passwordMaker) {
    if (autoPopulateCB.checked)
      passwordMaker.installListener();
    else
      passwordMaker.uninstallListener();
  }
}

function onPWShortcutLBChanged() {
  passwordMakerHelper.saveGlobalSetting("passwordShortcut", pwShortcutLB.value);
  passwordMakerHelper.datasourceFlusher.Flush();
  passwordMaker.passwordShortcut = pwShortcutLB.value;
}

function onMaskGeneratedPasswordButton() {
  if (!maskGeneratedPasswordCB.checked) {
    // User wants to turn off masking
    if (passwordMaster.value != "") {
      // A master password has been set, so we can't allow
      // them to turn off masking just yet
      // Firefox unchecked the checbkox, even though the user
      // hasn't entered the master pw yet. Check it here so
      // the UI matches the current state of affairs.
      maskGeneratedPasswordCB.checked = true;
      // Prompt user for the master password before turning off
      // generated password masking
      if (passwordMakerHelper.prompwMPW(passwordMaster.value))
        maskGeneratedPasswordCB.checked = false;
    }
  }
  passwordMakerHelper.saveGlobalSetting("maskMasterPassword", maskGeneratedPasswordCB.checked ? "true":"false");
  passwordMakerHelper.datasourceFlusher.Flush();
  passwordGenerated.setAttribute("type", maskGeneratedPasswordCB.checked ? "password" : "");
}