/**
  PasswordMaker - Creates and manages passwords
  Copyright (C) 2005 Eric H. Jung and LeahScape, Inc.
  http://passwordmaker.org/
  grimholtz@yahoo.com

  This library is free software; you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by
  the Free Software Foundation; either version 2.1 of the License, or (at
  your option) any later version.

  This library is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESSFOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
  for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this library; if not, write to the Free Software Foundation,
  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
**/

// References to the UI elements
var passwordMaster, passwordGenerated, passwordStorageLB, 
  passwordMakerHelper, createTreeFolderBtn, newAccountBtn, deleteTreeFolderOrItemBtn, 
  passwordGeneratedLabel, usingURLLabel, usingURLPrefixLabel;

var passwordMaker;

// State
var previousPasswordStorageLBValue, clipboardContents;

/**
 * Copy _str_ to the clipboard. If _str_ is "", the clipboard
 * is effectively cleared.
 */
function onCopyToClipboard(str) {
  const clipboardHelper = Components.classes["@mozilla.org/widget/clipboardhelper;1"].getService(Components.interfaces.nsIClipboardHelper);
  clipboardHelper.copyString(str);

  // Auto-clear the clipboard?
  if (passwordMaker._getLiteral(passwordMaker.globalSettingsSubjectRes, passwordMaker.autoClearClipboardPredicateRes) == "true") {
    // Yes. Store the data we've copied there so we know whether or not
    // something else has been copied into the clipboard since we put our data there.
    // Notice we're only storing it in this global var *iff* autoClearClipboard is true (more secure that way)
    clipboardContents = str;    
    //Get # of seconds
    var secs = passwordMaker._getLiteral(passwordMaker.globalSettingsSubjectRes, passwordMaker.autoClearClipboardSecondsPredicateRes);
    setTimeout("clearClipboard()", secs*1000)
  }
}

/**
 * Clear clipboard but only if
 * it still contains the data we placed there
 */
function clearClipboard() {
  try {
    var clip = Components.classes["@mozilla.org/widget/clipboard;1"].getService(Components.interfaces.nsIClipboard);
    if (!clip)
      return;

    var trans = Components.classes["@mozilla.org/widget/transferable;1"].createInstance(Components.interfaces.nsITransferable);
    if (!trans)
      return;
    trans.addDataFlavor("text/unicode");
    clip.getData(trans, clip.kGlobalClipboard);
    var str = new Object(), strLength = new Object();
    trans.getTransferData("text/unicode", str, strLength);
    if (str)
      str = str.value.QueryInterface(Components.interfaces.nsISupportsString);
    var clipContents;
    if (str) {
      clipContents = str.data.substring(0,strLength.value / 2);
      if (clipContents == clipboardContents) {
        // Clear the clipboard
        const clipboardHelper = Components.classes["@mozilla.org/widget/clipboardhelper;1"].getService(Components.interfaces.nsIClipboardHelper);
        clipboardHelper.copyString("");
      }
    }
  }
  catch (e) {
    // There's probably not text/unicode data
    // on the clipboard so let it be.
  }
  finally {
    clipboardContents = null; // Clear internal data member for proactive security
  }
}

function onShowMPWBtn() {
}

/**
 * Called when the dialog box is opening.
 */
function init() {
  passwordMaker = window.opener.passwordMaker;
  passwordMakerHelper = window.arguments[0].passwordMakerHelper;

  // Get references to the UI elements
  passwordMaster = document.getElementById("passwordMaster");
  passwordGenerated = document.getElementById("passwordGenerated");
  passwordStorageLB = document.getElementById("passwordStorageLB");
  createTreeFolderBtn = document.getElementById("createTreeFolderBtn");
  deleteTreeFolderOrItemBtn = document.getElementById("deleteTreeFolderOrItemBtn");
  passwordGeneratedLabel = document.getElementById("passwordGeneratedLabel");
  newAccountBtn = document.getElementById("newAccountBtn");
  usingURLLabel = document.getElementById("usingURLLabel");
  usingURLPrefixLabel = document.getElementById("usingURLPrefixLabel");

  //secretDecoderRing = Components.classes["@mozilla.org/security/sdr;1"].createInstance(Components.interfaces.nsISecretDecoderRing);

  // Load UI settings
  // passwordMaster.value = secretDecoderRing.decryptString(passwordMaster.getAttribute("saved"));
  if (passwordMakerHelper.hasMasterPassword()) {
    // User entered a master password when promtped from the context-menu or a pageLoad,
    // or it was stored as a preference and read at startup.
    passwordMaster.value = passwordMakerHelper.getDecryptedMasterPassword();
  }

  passwordMakerHelper.assertGlobals();
  // Set the UI to match the saved preferences
  passwordStorageLB.value = previousPasswordStorageLBValue = passwordMaker._getLiteral(passwordMaker.globalSettingsSubjectRes, passwordMaker.masterPasswordStoragePredicateRes);
  
  initAccountsPanel();
  initGlobalSettingsPanel();
  initDomainsPanel();

  var maskGeneratedPassword = passwordMaker._getLiteral(passwordMaker.globalSettingsSubjectRes, passwordMaker.maskMasterPasswordPredicateRes) == "true";
  passwordGenerated.setAttribute("type", maskGeneratedPassword ? "password" : "");
  generatepassword();

  installFiles();
  passwordMaster.focus();
}

/**
 * Called when user makes UI changes in the basic options dialog.
 */
function onBasicOptionsChange() {
  passwordMakerHelper.storeOrClearMasterPassword(passwordStorageLB.value, passwordMaster.value);
  passwordMakerHelper.saveGlobalSetting("masterPasswordStorage", passwordStorageLB.value);
  passwordMakerHelper.datasourceFlusher.Flush();
  generatepassword();
}

/**
 * Called when user makes UI changes in the advanced options dialog.
 * Persist them. Note the master pw, if the user wants to store it,
 * is stored with rijndael (AES) encryption.
 */
function onAdvancedOptionsChange() {
  if (passwordMaster.value == "") {
    if (passwordStorageLB == "store-on-disk") {
      passwordMakerHelper.prompts.alert(this, "PasswordMaker", "The encrypted master password has been removed from disk.");
      previousPasswordStorageLBValue = passwordStorageLB.value = "do-not-store";
    }
    maskMPWFieldCB.checked = false;
  }
  passwordMakerHelper.datasourceFlusher.Flush(); // don't think this is needed
  passwordMakerHelper.storeOrClearMasterPassword(passwordStorageLB.value, passwordMaster.value);

  // global-settings
  passwordMakerHelper.saveGlobalSetting("masterPasswordStorage", passwordStorageLB.value);
  passwordMakerHelper.datasourceFlusher.Flush();
}

function onPasswordStorageLBChanged() {
  // The actual storage or removal occurs in updateChanges(), called *after* this method
  if (previousPasswordStorageLBValue == "store-on-disk") {
    if (passwordStorageLB.value != "store-on-disk") {
      passwordMakerHelper.prompts.alert(this, "PasswordMaker", "The encrypted master password has been removed from disk.");
      passwordMaster.value = "";
    }
  }
  else {
    if (passwordStorageLB.value == "store-on-disk") {
      if (passwordMakerHelper.ask(this, "Although the master password is stored encrypted on disk,\nit is still a security risk to store it there because the\npassword can be decrypted fairly easily.\nAre you sure?")) {
        passwordMakerHelper.prompts.alert(this, "PasswordMaker", "You can remove the master password from disk\nby selecting one of the other storage options or clearing the master password field.");
      }
      else {
        // User canceled
        passwordStorageLB.value = previousPasswordStorageLBValue;
        return;
      }
    }
  }
  previousPasswordStorageLBValue = passwordStorageLB.value;
}

function generatepassword() {
  var cells = getSelectedTreeCells(accountsTree);
  passwordGenerated.value = passwordMakerHelper.generatepassword(
    cells[4].getAttribute("label"),  // hashAlgorithmLB,
    passwordMaster.value,
      getUrlToUse() + 
      cells[12].getAttribute("label") + // usernameTB
      cells[13].getAttribute("label"), // counter
    cells[2].getAttribute("label"), // whereLeetLB
    cells[3].getAttribute("label"), // leetLevelLB
    cells[5].getAttribute("label"), // passwordLength
    cells[14].getAttribute("label"), // charset
    cells[15].getAttribute("label"), // prefix
    cells[16].getAttribute("label")); // suffix
}

/**
 * Icon moving code (workaround for EM bug 242757).
 * This makes our icon appear on our dialog window
 * (otherwise the default icon is used).
 *
 * Windows And Linux only for now. Not sure what is needed for OSX.
 *
 * Thanks go out to asqueella and JedBrown for the QuickNote extension
 * and this help:
 * http://forums.mozillazine.org/viewtopic.php?t=219952
 */
function installFiles() {
  var iconFileName;
  var platform = navigator.platform.toLowerCase();

  if (platform.indexOf("win32") != -1)
    iconFileName = "passwordmakerdlg.ico";
  else if (platform.indexOf("linux") != -1)
    iconFileName = "passwordmakerdlg.xpm";
  else
    iconFileName = "passwordmakerdlg.ico"; // don't know what yet to do on Mac and other platforms

  // Target directory for icons is %MozDir%/chrome/icons/default/
  var destDir = PWDMKRDirIO.get("AChrom");
  destDir.append("icons");
  destDir.append("default");

  var destFile = destDir.clone();
  destFile.append(iconFileName);

  // Original file is at %Profile%/extensions/{5872365E-67D1-4AFD-9480-FD293BEBD20D}/defaults/_filename_
  var origDir = PWDMKRDirIO.get("ProfD");
  origDir.append("extensions");
  origDir.append("{5872365E-67D1-4AFD-9480-FD293BEBD20D}");
  origDir.append("defaults");
  var origFile = origDir.clone();
  origFile.append(iconFileName);
  installFile(destFile, destDir, origFile);
}

function installFile(destFile, destDir, origFile) {
  try {
    if (!destFile.exists() && origFile.exists())
      origFile.copyTo(destDir, null);
    if (!destFile.exists())
      passwordMakerHelper.prompts.alert(this, "PasswordMaker", "Unable to copy " + origFile + " to " + destDir + "\n\nDo you have sufficient read/write privileges?");
  }
  catch (e) {
    passwordMakerHelper.prompts.alert(this, "PasswordMaker", "Unable to copy " + origFile + " to " + destDir + "\n\nDo you have sufficient read/write privileges?");
    dump(e);
  }
}

function importSettings() {
  if (copyFile(false)) {
    //window.opener.setTimeout("passwordMaker.open()", 0);
    window.close();
  }
}

function exportSettings() {
  if (passwordMakerHelper.datasource.hasArcOut(passwordMaker.globalSettingsSubjectRes, passwordMaker.masterPasswordPredicateRes)) {
    if (!passwordMakerHelper.ask(this,
      "Since you've saved the master password to disk, it will also be exported. Although the master password is encrypted, it is safer not to export at this time. It is recommended that you first remove the master password from disk by clearing the master password field or selecting one of the other master password storage options. Do you want to continue anyway?"))
      return;
  }
  copyFile(true);
}

function copyFile(isExport) {

  var nsIFilePicker = Components.interfaces.nsIFilePicker;
  var fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
  fp.init(window, isExport?"Export Settings to File":"Import Settings from File", isExport?nsIFilePicker.modeSave:nsIFilePicker.modeOpen);
  fp.defaultExtension = "rdf";
  fp.appendFilters(nsIFilePicker.filterAll);
  var res = fp.show();
  if (res == nsIFilePicker.returnCancel)
    return;

  var f1 = Components.classes["@mozilla.org/file/local;1"]
    .createInstance(Components.interfaces.nsILocalFile);

  var f2 = Components.classes["@mozilla.org/file/local;1"]
    .createInstance(Components.interfaces.nsILocalFile);

  if (!fp.file || !f1 || !f2)
    passwordMakerHelper.prompts.alert(this, "PasswordMaker", "An error occured.");
  
  var pwmkrRDF = PWDMKRDirIO.get("ProfD"); // %Profile% dir
  pwmkrRDF.append("passwordmaker.rdf");

  try {
    f1.initWithFile(pwmkrRDF);
    if (isExport) {
      if (fp.file.exists())
        fp.file.remove(false);
      f2.initWithPath(fp.file.path.substring(0, fp.file.path.indexOf(fp.file.leafName)));
      f1.copyTo(f2, fp.file.leafName);
    }
    else {
      if (!validateImportFile(fp.file)) {
        passwordMakerHelper.prompts.alert(this, "PasswordMaker", fp.file.path + " is not a valid PasswordMaker file. Import canceled.");
        return false;
      }

      if (!passwordMakerHelper.ask(this,
          "All existing settings (accounts, account settings, global settings, special domains, etc.) will be overwritten. ARE YOU SURE YOU WANT TO IMPORT THIS FILE?"))
          return false;

      if (pwmkrRDF.exists())
        pwmkrRDF.remove(false);
      f2.initWithPath(fp.file.path.substring(0, fp.file.path.indexOf(fp.file.leafName)));
      fp.file.copyTo(PWDMKRDirIO.get("ProfD"), "passwordmaker.rdf");
    }
    passwordMakerHelper.prompts.alert(this, "PasswordMaker",
      isExport ? 
      ("Settings have been exported to " + fp.file.path + ". The file can be imported on other computers using PasswordMaker's File->Import option") :
      ("Settings have been imported from " + fp.file.path + ". Please restart all browsers for the new settings to take effect.")); 
    return true;
  }
  catch (e) {
    dump(e);
    passwordMakerHelper.prompts.alert(this, "PasswordMaker", "An error occured and " + f1.path +
      " could not be copied to " + fp.file.path + ". Do you have write access to that directory?");
    return false;
  }
}

function validateImportFile(file) {
  try {
    var ioService = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
		var datasourceURI = ioService.newFileURI(file).spec;
    var datasource = passwordMakerHelper.rdfService.GetDataSourceBlocking(datasourceURI);
    var datasourceFlusher = datasource.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource);
    var accountsSubjectRes = passwordMakerHelper.rdfService.GetResource("http://passwordmaker.mozdev.org/accounts");
    return !passwordMakerHelper.rdfContainerUtils.IsEmpty(datasource, accountsSubjectRes);
    //TODO: even more validation
  }
  catch (e) {
    dump(e);
    return false;
  }
}

/**
 * Open the proper options dialog and close this one.
 */
function onOptionsDialogButton(whichOptionsDialog) {
  if (window.opener && passwordMaker)
    passwordMaker.switchOptionsDialog();  
  window.close();
}

function updateURLLabel() {
  // Update the "Using URL" label
  var url = getUrlToUse();
  if (!url || url == "")
    url = "[none - click account settings to set]";
  usingURLPrefixLabel.value = "Using URL ";
  usingURLLabel.value = url;
}